use weave::Error;



/// The errors that may occur during conversion from an array of bytes
/// to an actual value.
#[derive(Copy, Clone, Eq, PartialEq)]
pub enum ConversionError
{
   /// A corruption of the binary data has been detected.
   CorruptionDetected,

   /// An invalid value was detected.
   InvalidValue,

   /// There were not enough bytes to create the desired type.
   TooFewBytes,

   /// There were to many bytes to create the desired type.
   TooManyBytes,
}



impl Error for ConversionError
{
   fn get_description(&self) -> &str
   {
      match *self
      {
         ConversionError::CorruptionDetected =>
         {
            "A corruption of the byte data was detected."
         }

         ConversionError::InvalidValue =>
         {
            "An invalid value was transmuted from the given byte data."
         }

         ConversionError::TooFewBytes =>
         {
            "Not enough bytes of data were provided \
             to properly create this type."
         }

         ConversionError::TooManyBytes =>
         {
            "Too many bytes of data were provided \
             to properly create this type."
         }
      }
   }

   fn get_cause(&self) -> Option<&Error>
   {
      match *self
      {
         _ =>
         {
            None
         }
      }
   }
}

impl ::std::fmt::Debug for ConversionError
{
   fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result
   {
      ::std::fmt::Display::fmt(self, f)
   }
}

impl ::std::fmt::Display for ConversionError
{
   fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result
   {
      match *self
      {
         ConversionError::CorruptionDetected =>
         {
            write!(f, "Byte Conversion Error:  CorruptionDetected")
         }

         ConversionError::InvalidValue =>
         {
            write!(f, "Byte Conversion Error:  InvalidValue")
         }

         ConversionError::TooFewBytes =>
         {
            write!(f, "Byte Conversion Error:  TooFewBytes")
         }

         ConversionError::TooManyBytes =>
         {
            write!(f, "Byte Conversion Error:  TooManyBytes")
         }
      }
   }
}
